/*
 * Style tweaks
 * --------------------------------------------------
 */
html {
  overflow-x: hidden; /* Prevent scroll on narrow devices */
}
body {
  padding-top: 70px;
}
footer {
  padding: 30px 0;
}

ul#slides li {
  list-style-type: none;
}

ul#slides {
  margin: 0;
  padding: 0;
}

ul#slides li div div {
  padding:0 0.2em;
}

.mainmenu {
        height: 80px;
        width : 100%;
}

.linegraph {
        width : 900px;
        height: 400px;
        background:#FFFFFF;
        border:4px double white;
        padding:0 10px;
        margin: 35px auto;
}
.piegraph {
	width : 500px;
	height: 250px;
	background:#FFFFFF;
	border:4px double white;
	padding:0 10px;
	margin: 35px auto;
}

.smallgraph {
        width : 600px;
        height: 150px;
        background:#FFFFFF;
        border:4px double white;
        padding:0 10px;
        margin: 35px auto;
}
.dropdown {font-size:1.2em;}
.dropdown-submenu{position:relative;}
.dropdown-submenu>.dropdown-menu{top:0;left:100%;margin-top:-6px;margin-left:-1px;-webkit-border-radius:0 6px 6px 6px;-moz-border-radius:0 6px 6px 6px;border-radius:0 6px 6px 6px;}
.dropdown-submenu:hover>.dropdown-menu{display:block;}
.dropdown-submenu>a:after{display:block;content:" ";float:right;width:0;height:0;border-color:transparent;border-style:solid;border-width:5px 0 5px 5px;border-left-color:#cccccc;margin-top:5px;margin-right:-10px;}
.dropdown-submenu:hover>a:after{border-left-color:#ffffff;}
.dropdown-submenu.pull-left{float:none;}.dropdown-submenu.pull-left>.dropdown-menu{left:-100%;margin-left:10px;-webkit-border-radius:6px 0 6px 6px;-moz-border-radius:6px 0 6px 6px;border-radius:6px 0 6px 6px;}

.panel { background-color: #f5f5f5;}
.panel-body { background-color: #ffffff; font-size: 1.2em;}
.panel-body a {        margin: 5px; }
.panel-footer { background-color: #f5f5f5}
.panel-height { height: 300px}
.panel-xlheight { height: 750px}
.btn { margin-top: 10px};
.key-figures ul {
	margin: 0;
	padding: 0;
}
.key-figures li {
	list-style-type: none;
	margin-bottom: 0.5em;
}
.figure {
	font-weight: bold;
	font-size: 1.0em;
	color:#c43c35;
}
.figure-label {
	color: #666;
	font-size: 0.9em;
}
.figure-date {
	color: #333;
	font-size: 0.8em;
}
.mouse-figures {
	font-weight: bold;
	font-size: 1.0em;
	margin: 0;
	padding: 0;
}       

.mfigure {
	font-weight: bold;
	font-size: 1.2em;
	color: #d9534f;
}

.mfigure small {
	font-weight: bold;
	font-size: 0.8em;
	color:#5f5555;
}

.span11 {
	font-size: 0.8em;
}

/*
&.danger {
    .gradientBar(#ee5f5b, #c43c35);
}
*/
/*
 * Off Canvas
 * --------------------------------------------------
 */
@media screen and (max-width: 767px) {
  .row-offcanvas {
    position: relative;
    -webkit-transition: all 0.25s ease-out;
    -moz-transition: all 0.25s ease-out;
    transition: all 0.25s ease-out;
  }

  .row-offcanvas-right
  .sidebar-offcanvas {
    right: -50%; /* 6 columns */
  }

  .row-offcanvas-left
  .sidebar-offcanvas {
    left: -50%; /* 6 columns */
  }

  .row-offcanvas-right.active {
    right: 50%; /* 6 columns */
  }

  .row-offcanvas-left.active {
    left: 50%; /* 6 columns */
  }

  .sidebar-offcanvas {
    position: absolute;
    top: 0;
    width: 50%; /* 6 columns */
  }
}

.jqplot-graph { color: #ac1316; z-index: 99;}
.jqplot-target {
    position: relative;
    color: #333;
    font-size: 0.8em;
}

.jqplot-highlighter-tooltip {
    border: 0px;
    font-size: 1.0em;
    white-space: nowrap;
    font-weight: bold;
    margin: 5px;
    padding: 5px;
    color:#5f5555;
    background: #f5f5f5;
    border:1px solid #DCA;
    -webkit-border-radius:6px;
    -moz-border-radius:6px;
    border-radius:6px;
    -webkit-box-shadow:0 5px 10px rgba(0, 0, 0, 0.2);
    -moz-box-shadow:0 5px 10px rgba(0, 0, 0, 0.2);
    box-shadow:0 5px 10px rgba(0, 0, 0, 0.2);
}

 .jqplot-title {
    font-size: 1.1em;
    text-transform: uppercase;
    letter-spacing: .1em;
    margin: 0;
    padding: 0;
    color: #5f5555;
 }

 #start-date {
    color: #000000;
 }

 #end-date {
    color: #000000;
 }


#pgcluuModal .modal-dialog 
{ 
     height: 100%; 
     width: 1200px; 
     overflow-y: hidden; 
}

.btn-danger {
	background: #5f5555;
	color: #f5f5f5;
    border:1px solid #DCA;
}

